#include "StdAfx.h"
#include "Definitions.h"

#define _USE_MATH_DEFINES

// For Cordic calculations.
extern WORD HalfBase;

void DrawCross(HDC hDC, POINT pt, COLORREF colr, int size)
/*
  USE:  Draw cross on screen at pt with given color.
*/
{
  // Save initial pen.
  HGDIOBJ penInit = SelectObject(hDC, GetStockObject(DC_PEN));
  SetDCPenColor(hDC, colr);

  // Horizontal arm.
  MoveToEx(hDC, pt.x - size, pt.y, (LPPOINT) NULL); 
  LineTo(hDC, pt.x + size + 1, pt.y);

  // Vertical arm.
  MoveToEx(hDC, pt.x, pt.y + size, (LPPOINT) NULL); 
  LineTo(hDC, pt.x, pt.y - size - 1);

  // Restore initial pen
  SelectObject(hDC, penInit);
}

void CalcHexPtsCordic(POINT center, POINT vertex, POINT* pPts, int count)
/*
  USE:  Calculate array of hex vertices using Cordic calcs.
  IN:   center = center of hexagon.
        vertex = one of the hexagon vertices
  NOTE: Loads buffer with the given vertex and the other five vertices of a regular hexagon.
        Uses CORDIC routines for trig and long integer calculations.
*/
{
  int   sine;     // sine of central angle
  int   cosine;   // cosine of central angle
  POINT ptDelta;   // vertex - center (radial spoke)

  // 60 deg = 10923 Cordic units.
  SinCos(10923, &sine, &cosine);

  // Set initial and final point to incoming vertex.
  pPts[0].x = pPts[count-1].x = vertex.x;
  pPts[0].y = pPts[count-1].y = vertex.y;

  // Go clockwise around circle to calculate hex points.
  for (int corner = 1; corner < count-1; corner++)
  {
    // Calculate the radial spoke : vertex - center.
    ptDelta.x = vertex.x - center.x;
    ptDelta.y = vertex.y - center.y;

    vertex.x = (int)(((long) ptDelta.x * cosine - (long) ptDelta.y * sine + HalfBase) >> NBITS) + center.x;
    vertex.y = (int)(((long) ptDelta.x * sine + (long) ptDelta.y * cosine + HalfBase) >> NBITS) + center.y;

    // Store new vertex in array.
    pPts[corner].x = vertex.x;
    pPts[corner].y = vertex.y;
  } // for
}

POINT GetVectorCordic(POINT p, int angleCordic, double distance)
{
  POINT q;
  int sine, cosine;

  SinCos(angleCordic, &sine, &cosine);
  q.x = p.x + (int)(((long) distance * cosine + HalfBase) >> NBITS);
  q.y = p.y + (int)(((long) distance * sine + HalfBase) >> NBITS);

  return q;
}

void DrawEllipse(HDC hDC, POINT pCenter, int radius)
{
  // Save initial brush.
  HGDIOBJ brushInit = SelectObject(hDC, GetStockObject(NULL_BRUSH));

  int xLeft = pCenter.x - radius;
  int xRight = pCenter.x + radius;
  int yTop = pCenter.y - radius;
  int yBottom = pCenter.y + radius;

  Ellipse(hDC, xLeft, yTop, xRight, yBottom);

  // Restore initial brush;
  SelectObject(hDC, brushInit);

}

void DrawVector(HDC hDC, POINT p, POINT q, COLORREF colr)
{
  DrawCross(hDC, p, colBlack, 4);

  // Save initial pen.
  HGDIOBJ penInit = SelectObject(hDC, GetStockObject(DC_PEN));

  SetDCPenColor(hDC, colr);

  MoveToEx(hDC, p.x, p.y, (LPPOINT) NULL); 
  LineTo(hDC, q.x, q.y);

  // Restore initial pen
  SelectObject(hDC, penInit);
}

void DrawVectorAtAngle(HDC hDC, POINT p, double angleDeg, int distance, int colr)
{
  int angleCau = GetAngleCau(angleDeg);
  POINT q = GetVectorCordic(p, angleCau, distance);
  DrawVector(hDC, p, q, colr);
}
  
 void DrawHexagon(HDC hDC, POINT center, POINT vertex)
/*
  USE:  Draw Hexagon given center and one vertex.
  IN:   center = center of hexagon.
        vertex = one of the hexagon vertices
  NOTE: Call CalcHexPtsCordic() to load array with hexagon vertices.
*/
{
  POINT pts[7];

  CalcHexPtsCordic(center, vertex, pts, 7);
  Polyline(hDC, pts, 7);
}
